@@ -1,31 +1,12 @@
 {-# LANGUAGE CPP #-}
-#ifndef MIN_VERSION_base
--- These are not precise, but allow the module to be built without cabal.
-#if __GLASGOW_HASKELL__ >= 709
-#define MIN_VERSION_base(major1,major2,minor) (((major1) <= 4) && ((major2) <= 8))
-#elif __GLASGOW_HASKELL__ >= 707
-#define MIN_VERSION_base(major1,major2,minor) (((major1) <= 4) && ((major2) <= 7))
-#elif __GLASGOW_HASKELL__ >= 706
-#define MIN_VERSION_base(major1,major2,minor) (((major1) <= 4) && ((major2) <= 6))
-#else
-#define MIN_VERSION_base(major1,major2,minor) 0
-#endif
-#endif
-
-#ifdef __GLASGOW_HASKELL__
-#define OVERLOADED_STRINGS 1
-#endif
-
-#ifdef OVERLOADED_STRINGS
-{-# LANGUAGE OverloadedStrings #-}
-#endif
+#include "boxes.h"
 
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Text.PrettyPrint.Boxes
 -- Copyright   :  (c) Brent Yorgey 2009
 -- License     :  BSD-style (see LICENSE)
--- Maintainer  :  byorgey@cis.upenn.edu
+-- Maintainer  :  David.Feuer@gmail.com
 -- Stability   :  experimental
 -- Portability :  portable
 --
@@ -97,9 +78,15 @@ module Text.PrettyPrint.Boxes
 
     ) where
 
-#if MIN_VERSION_base(4,8,0)
+#if MIN_VERSION_base(4,11,0)
+import Prelude hiding ( (<>), Word )
+#elif MIN_VERSION_base(4,8,0)
 import Prelude hiding (Word)
+#else
+import Data.Foldable (Foldable (foldr))
+import Prelude hiding (foldr)
 #endif
+import Data.Foldable (toList)
 
 #if MIN_VERSION_base(4,4,0)
 import Data.String (words, unwords)
@@ -213,35 +200,46 @@ t // b = vcat left [t,b]
 t /+/ b = vcat left [t, emptyBox 1 0, b]
 
 -- | Glue a list of boxes together horizontally, with the given alignment.
-hcat :: Alignment -> [Box] -> Box
-hcat a bs = Box h w (Row $ map (alignVert a h) bs)
-  where h = maximum . (0:) . map rows $ bs
-        w = sum . map cols $ bs
+hcat :: Foldable f => Alignment -> f Box -> Box
+hcat a bs = Box h w (Row $ map (alignVert a h) bsl)
+  where
+    (w, h) = sumMax cols 0 rows bsl
+    bsl = toList bs
 
 -- | @hsep sep a bs@ lays out @bs@ horizontally with alignment @a@,
 --   with @sep@ amount of space in between each.
-hsep :: Int -> Alignment -> [Box] -> Box
+hsep :: Foldable f => Int -> Alignment -> f Box -> Box
 hsep sep a bs = punctuateH a (emptyBox 0 sep) bs
 
 -- | Glue a list of boxes together vertically, with the given alignment.
-vcat :: Alignment -> [Box] -> Box
-vcat a bs = Box h w (Col $ map (alignHoriz a w) bs)
-  where h = sum . map rows $ bs
-        w = maximum . (0:) . map cols $ bs
+vcat :: Foldable f => Alignment -> f Box -> Box
+vcat a bs = Box h w (Col $ map (alignHoriz a w) bsl)
+  where
+    (h, w) = sumMax rows 0 cols bsl
+    bsl = toList bs
+
+-- Calculate a sum and a maximum over a list in one pass. If the list is
+-- empty, the maximum is reported as the given default. This would
+-- normally be done using the foldl library, but we don't want that
+-- dependency.
+sumMax :: (Num n, Ord b, Foldable f) => (a -> n) -> b -> (a -> b) -> f a -> (n, b)
+sumMax f defaultMax g as = foldr go (,) as 0 defaultMax
+  where
+    go a r n b = (r $! f a + n) $! g a `max` b
 
 -- | @vsep sep a bs@ lays out @bs@ vertically with alignment @a@,
 --   with @sep@ amount of space in between each.
-vsep :: Int -> Alignment -> [Box] -> Box
-vsep sep a bs = punctuateV a (emptyBox sep 0) bs
+vsep :: Foldable f => Int -> Alignment -> f Box -> Box
+vsep sep a bs = punctuateV a (emptyBox sep 0) (toList bs)
 
 -- | @punctuateH a p bs@ horizontally lays out the boxes @bs@ with a
 --   copy of @p@ interspersed between each.
-punctuateH :: Alignment -> Box -> [Box] -> Box
-punctuateH a p bs = hcat a (intersperse p bs)
+punctuateH :: Foldable f => Alignment -> Box -> f Box -> Box
+punctuateH a p bs = hcat a (intersperse p (toList bs))
 
 -- | A vertical version of 'punctuateH'.
-punctuateV :: Alignment -> Box -> [Box] -> Box
-punctuateV a p bs = vcat a (intersperse p bs)
+punctuateV :: Foldable f => Alignment -> Box -> f Box -> Box
+punctuateV a p bs = vcat a (intersperse p (toList bs))
 
 --------------------------------------------------------------------------------
 --  Paragraph flowing  ---------------------------------------------------------
@@ -289,7 +287,7 @@ getLines (Para _ (Block ls l))
 data Line = Line { lLen :: Int, getWords :: [Word] }
 
 mkLine :: [Word] -> Line
-mkLine ws = Line (sum (map wLen ws) + length ws - 1) ws
+mkLine ws = Line (sum (map ((+1) . wLen) ws) - 1) ws
 
 startLine :: Word -> Line
 startLine = mkLine . (:[])
@@ -318,13 +316,13 @@ wordFits pw w l = lLen l == 0 || lLen l + wLen w + 1 <= pw
 --   contents and height of @bx@, horizontally aligned according to
 --   @algn@.
 alignHoriz :: Alignment -> Int -> Box -> Box
-alignHoriz a c b = Box (rows b) c $ SubBox a AlignFirst b
+alignHoriz a c b = align a AlignFirst (rows b) c b
 
 -- | @alignVert algn n bx@ creates a box of height @n@, with the
 --   contents and width of @bx@, vertically aligned according to
 --   @algn@.
 alignVert :: Alignment -> Int -> Box -> Box
-alignVert a r b = Box r (cols b) $ SubBox AlignFirst a b
+alignVert a r b = align AlignFirst a r (cols b) b
 
 -- | @align ah av r c bx@ creates an @r@ x @c@ box with the contents
 --   of @bx@, aligned horizontally according to @ah@ and vertically
